"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getWebpackConfig = void 0;
const path = __importStar(require("path"));
const os = __importStar(require("os"));
const sass_1 = __importDefault(require("sass"));
const lodash_1 = require("lodash");
const terser_webpack_plugin_1 = __importDefault(require("terser-webpack-plugin"));
const mini_css_extract_plugin_1 = __importDefault(require("mini-css-extract-plugin"));
const webpack_bundle_analyzer_1 = require("webpack-bundle-analyzer");
const webpack_1 = __importStar(require("webpack"));
const fork_ts_checker_webpack_plugin_1 = __importDefault(require("fork-ts-checker-webpack-plugin"));
const get_css_loader_1 = require("./get-css-loader");
const fs_extra_1 = __importDefault(require("fs-extra"));
const package_json_1 = __importDefault(require("../../../package.json"));
const VueLoaderPlugin = require('vue-loader/lib/plugin-webpack5');
const ReactRefreshWebpackPlugin = require('@pmmmwh/react-refresh-webpack-plugin');
function getWebpackConfig(info) {
    const { name, context, entry, output, mode, libraryTarget, isTs, splitChunks, publicPath, sourceMap, experimental } = info;
    const prodSourceMap = sourceMap ? 'source-map' : undefined;
    const isProd = mode === 'production';
    const isUseSwc = (0, lodash_1.get)(experimental, 'swc', false) || !(isProd || os.platform() === 'win32');
    const corejsPkgJsonPath = require.resolve('core-js/package.json');
    const corejsVersion = fs_extra_1.default.readJSONSync(corejsPkgJsonPath).version;
    const plugins = [
        new VueLoaderPlugin(),
        new webpack_1.IgnorePlugin({
            resourceRegExp: /^\.\/locale$/,
            contextRegExp: /moment$/,
        }),
    ];
    if (isTs) {
        plugins.push(new fork_ts_checker_webpack_plugin_1.default({
            typescript: {
                typescriptPath: require.resolve('typescript'),
            },
        }));
    }
    plugins.push(new mini_css_extract_plugin_1.default({ filename: `${name}.[name].css` }));
    plugins.push(new webpack_1.ProvidePlugin({
        wedaConfigTransfer: [path.resolve(__dirname, 'transfer'), 'default'],
    }));
    if (!splitChunks) {
        // https://github.com/webpack/webpack/issues/3637 坑
        plugins.push(new webpack_1.default.optimize.LimitChunkCountPlugin({
            maxChunks: 1,
        }));
    }
    if (isProd) {
        // 生产插件
        if (!!process.env.ANALYZER && process.env.ANALYZER !== 'false') {
            plugins.push(new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
                analyzerMode: 'static',
            }));
        }
    }
    else {
        plugins.push(new ReactRefreshWebpackPlugin());
    }
    const rules = [
        {
            test: /\.vue$/,
            loader: require.resolve('vue-loader'),
        },
    ];
    if (!isProd) {
        rules.push({
            test: /\.json$/,
            use: {
                loader: require.resolve('./loaders/image2base64'),
            },
        });
    }
    if (isTs && !isUseSwc) {
        // window 的debug 也用 ts-loader
        // 其他的生产环境监测有ts文件才使用ts-loader
        rules.push({
            test: /.tsx?$/,
            exclude: /node_modules\/(?!@cloudbase\/weda-ui)|gsd-kbone-react/,
            use: [
                {
                    loader: require.resolve('ts-loader'),
                    options: {
                        transpileOnly: true,
                        getCustomTransformers: () => ({
                            before: [!isProd ? require('react-refresh-typescript')() : false].filter(Boolean),
                        }),
                    },
                },
            ],
        });
    }
    if (!isUseSwc) {
        rules.push({
            test: /\.(js|jsx)$/,
            // https://stackoverflow.com/questions/58489738/minicssextractplugin-error-on-entry-point-build
            exclude: [
                /node_modules[\\\/](css-loader|core-js|promise-polyfill|webpack|html-webpack-plugin|whatwg-fetch)[\\\/]/,
            ],
            use: [
                {
                    loader: require.resolve('babel-loader'),
                    options: {
                        compact: false,
                        sourceType: 'unambiguous',
                        presets: [
                            [
                                require.resolve('@babel/preset-env'),
                                {
                                    modules: false,
                                    targets: 'defaults and not ie <= 11',
                                    useBuiltIns: 'usage',
                                    corejs: corejsVersion,
                                },
                            ],
                            require.resolve('@babel/preset-react'),
                        ],
                        plugins: [
                            require.resolve('@babel/plugin-proposal-class-properties'),
                            [require.resolve('@babel/plugin-proposal-decorators'), { legacy: true }],
                            require.resolve('@babel/plugin-proposal-export-default-from'),
                            require.resolve('@babel/plugin-proposal-export-namespace-from'),
                            require.resolve('@babel/plugin-proposal-optional-chaining'),
                            [
                                require.resolve('@babel/plugin-transform-runtime'),
                                {
                                    absoluteRuntime: path.dirname(require.resolve('@babel/runtime/package.json')),
                                },
                            ],
                        ].filter(Boolean),
                    },
                },
            ],
        });
    }
    else {
        // 开发环境且不是window 使用swc
        rules.push({
            test: /\.(j|t)sx?$/,
            // https://stackoverflow.com/questions/58489738/minicssextractplugin-error-on-entry-point-build
            exclude: [
                /node_modules[\\\/](css-loader|core-js|promise-polyfill|webpack|html-webpack-plugin|whatwg-fetch)[\\\/]/,
            ],
            loader: require.resolve('swc-loader'),
            options: {
                jsc: {
                    target: 'es2021',
                    parser: {
                        syntax: 'typescript',
                        jsx: true,
                        tsx: true,
                    },
                    transform: {
                        react: {
                            refresh: true,
                        },
                    },
                },
            },
        });
    }
    rules.push({
        test: /\.css$/,
        use: (0, get_css_loader_1.getCssLoader)(info),
    });
    rules.push({
        test: /\.less$/,
        use: [
            ...(0, get_css_loader_1.getCssLoader)(info),
            {
                loader: require.resolve('less-loader'),
                options: {
                    lessOptions: {
                        javascriptEnabled: true,
                    },
                },
            },
        ],
    });
    rules.push({
        test: /\.(scss|sass)$/,
        use: [
            ...(0, get_css_loader_1.getCssLoader)(info),
            {
                loader: require.resolve('sass-loader'),
                options: {
                    implementation: sass_1.default,
                },
            },
        ],
    });
    rules.push({
        test: /\.(jpe?g|png|gif|ttf|eot|svg|woff(2)?)(\?[a-z0-9=&.]+)?$/,
        loader: require.resolve('base64-inline-loader'),
    });
    const configVars = JSON.stringify(info);
    const pkgVersion = package_json_1.default.version;
    return {
        name,
        context,
        mode,
        entry,
        externals: {
            // 兼容配置
            react: 'React',
            'react-dom': 'ReactDom',
            'react-dom/server': 'ReactDomServer',
            mobx: 'Mobx',
            'mobx-react-lite': 'MobxReactLite',
        },
        output: {
            path: path.resolve(context, output),
            filename: `${name}.[name].js`,
            library: {
                name: `@weapps-materials-[name]-${name}`,
                type: libraryTarget || 'window',
                umdNamedDefine: true,
            },
            publicPath,
        },
        resolve: {
            modules: ['node_modules', path.resolve(process.cwd(), 'node_modules'), path.resolve(context, 'node_modules')],
            extensions: ['.js', '.jsx', '.tsx', '.ts', '.json', '.scss', '.less', '.css'],
            alias: {
                'core-js': path.dirname(corejsPkgJsonPath),
            },
            fallback: {
                /** for @antv/g6@3.8.5 */
                util: require.resolve('node-util-polyfill'),
            },
        },
        cache: {
            type: 'filesystem',
            version: `${pkgVersion}|${configVars}`,
        },
        optimization: {
            splitChunks: splitChunks
                ? {
                    chunks: 'async',
                    minSize: 1024 * 1024 * 2,
                    cacheGroups: {
                        base: {
                            test: /(react|react-dom|react-router|react-router-dom|mobx|mobx-react-lite|@cloudbase\/js-sdk)/,
                            minSize: 500000,
                            priority: 100, // 优先级
                        },
                        utils: {
                            test: /(lodash|dayjs|moment|axios|kbone-api|fastclick)/,
                            priority: 100, // 优先级
                        },
                        'async-commons': {
                            minChunks: 2,
                            priority: 20,
                        },
                        commons: {
                            minChunks: 2,
                            priority: 20,
                        },
                    },
                }
                : false,
            minimize: true,
            minimizer: [
                new terser_webpack_plugin_1.default({
                    minify: terser_webpack_plugin_1.default.swcMinify,
                }),
            ],
        },
        devtool: isProd ? prodSourceMap : 'eval-cheap-source-map',
        plugins,
        module: {
            rules: rules.filter(Boolean),
        },
    };
}
exports.getWebpackConfig = getWebpackConfig;
